#!/usr/bin/python

#    gmail.sh - A gmail (or IMAP/SMTP) client to read and send your mails direct from console
#    http://www.ldlabs.org/ - http://www.francescoapollonio.com/
#    Mail: f.apollonio@ldlabs.org

#    Created By Francesco Apollonio - Copyright 2011 Francesco Apollonio
#
#    http://www.gnu.org/licenses/gpl.txt
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import imaplib, getpass, smtplib
import re
import sys
import signal, os
import email
import readline, atexit

list_response_pattern = re.compile(r'\((?P<flags>.*?)\) "(?P<delimiter>.*)" (?P<name>.*)')
list_status_pattern = re.compile(r'"(?P<mailbox>.*)" \(UNSEEN (?P<unseen>[0-9]{1,})\)')

def ctrl_c(signum, frame):
	#readline.__str__ = ""
	#readline.redisplay()
	return

class Credential(object):

	def __init__(self, username=""):
		self.username=username
		self.password=""

	def get_username(self):
		return self.username

	def get_password(self):
		if (self.password == ""):
			self.ask_password()
		return self.password

	def ask_password(self):
		self.password=getpass.getpass()

class MySMTP(object):

	#def __init__(self, user="", password="", server="", port=-1):
	#	self.username=user
	#	self.password=password
	def __init__(self, server="", port=-1, credential=None):
		self.server=server
		self.port=port
		self.smtp = None
		self.connected=False
		self.credential=credential

	def __del__(self):
		self.disconnect()

	def login_data(self):
		if self.credential is not None:
			self.ask_password()
		else:
			user = raw_input("Username: ")
			self.credential = Credential(user)
			self.credential.ask_password()

	def connect(self):
		if self.connected != False:
			return

		while self.connected == False:
			if self.credential is not None and self.credential.get_username() != "" and self.credential.get_password() != "" and self.server != "" and self.port != -1:
				try:
					self.smtp = smtplib.SMTP_SSL(self.server, self.port)
					self.smtp.login(self.credential.get_username(), self.credential.get_password())
					self.connected=True
				except Exception as ex:
					self.login_data()
			else:
				self.login_data()

	def disconnect(self):
		try:
			if self.smtp != None:
				self.smtp.close()
				print "Disconnected"
		except Exception:
			print "Disconnected"

	def send_mail(self, args):
		if (len(args) != 2):
			return
		self.connect()
		fromaddr = self.credential.get_username()
		toaddrs = args[0]
		subject = args[1]
		print "Enter message, end with ^D (Unix) or ^Z (Windows):"

		# Add the From: and To: headers at the start!
		msg = ("From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (fromaddr, toaddrs, subject))

		while 1:
    			try:
        			line = raw_input()
    			except EOFError:
        			break
			if not line:
				line = "\n"
			msg = msg + line
		self.smtp.sendmail(fromaddr, toaddrs, msg)
		print "SENT"

class MyImap(object):

	#def __init__(self, user="", password="", server="", port=-1):
	def __init__(self,server="", port=-1, credential=None):
		self.server=server
		self.port=port
		self.imap = None
		self.connected=False
		self.mailbox="INBOX"
		self.credential = credential

	def __del__(self):
		self.disconnect()

	def login_data(self):
		if self.credential is not None:
			self.credential.ask_password()
		else:
			user = raw_input("Username: ")
			self.credential = Credential(user)
			self.credential.ask_password()

	def connect(self):
		if self.connected != False:
			return

		while self.connected == False:
			if self.credential is not None and self.credential.get_username() != "" and self.credential.get_password() != "" and self.server != "" and self.port != -1:
				try:
					self.imap = imaplib.IMAP4_SSL(self.server, self.port)
					self.imap.login(self.credential.get_username(), self.credential.get_password())
					self.connected=True
				except Exception as ex:
					self.login_data()
			else:
				self.login_data()

	def unread(self, mailbox=None):
		if mailbox==None:
			mailbox=self.mailbox
		else:
			mailbox=mailbox[0].strip()
		self.connect()
		status= self.imap.status(mailbox, '(UNSEEN)')
		mailbox, unseen = list_status_pattern.match(status[1][0]).groups()
		print "Mailbox %s: %s" % (mailbox, unseen)

	def list_all(self):
		self.connect()
		typ, data = self.imap.list()
		for line in data:
			flags, delimiter, mailbox_name = list_response_pattern.match(line).groups()
			status = self.imap.status(mailbox_name, '(UNSEEN)')
			if status[0] != "NO":
				mailbox, unseen = list_status_pattern.match(status[1][0]).groups()
				print "Mailbox %s: %s" % (mailbox, unseen)
			else:
				print "Mailbox %s: ERROR" %mailbox_name

	def list_messages(self, args=None):
		mailbox=""
		search=""
		if args==None:
                        mailbox=self.mailbox
			search = "UNSEEN"
                elif len(args) == 1:
                        mailbox=args[0].strip()
			search = "UNSEEN"
		elif len(args) == 2:
			mailbox = args[1].strip()
			search = args[0].strip()
		print "Messages in %s" % mailbox
		self.connect()
		self.imap.select(mailbox, readonly=True)
		typ, msg_ids = self.imap.search("UTF-8", "("+search+")")
		for message in msg_ids[0].split():
			typ, msg_data = self.imap.fetch(message, '(BODY[HEADER.FIELDS (SUBJECT FROM)])')
			for response_part in msg_data:
        			if isinstance(response_part, tuple):
            				msg = email.message_from_string(response_part[1])
					print "[ID: %s] From: %s Subject: %s" % (message, msg['from'], msg['subject'])

	def view_message(self, args):
		mailbox=""
		id=""
		if len(args) == 2:
			mailbox = args[1].strip()
			id = args[0].strip()
		elif len(args) == 1:
			mailbox=self.mailbox
			id = args[0]

		self.imap.select(mailbox, readonly=True)
		typ, msg_data = self.imap.fetch(id, '(BODY[HEADER.FIELDS (SUBJECT FROM TO)])')
		for response_part in msg_data:
        		if isinstance(response_part, tuple):
            			msg = email.message_from_string(response_part[1])
				print "[ID: %s] From: %s To: %s\nSubject: %s" % (id, msg['from'], msg['to'], msg['subject'])
		print "Body Text:\n"
		typ, msg_data = self.imap.fetch(id, '(BODY[TEXT])')
		for response_part in msg_data:
			if isinstance(response_part, tuple):
				print response_part[1]

	def set_mailbox(self, args):
		if len(args) == 1:
			self.mailbox = args[0]

	def disconnect(self):
		try:
			if self.imap != None:
				self.imap.close()
				self.imap.logout()
				print "Disconnected"
		except Exception:
			print "Disconnected"


def help():
	print "\tunread [MAILBOX]: lists unreaded messages from the specified MAILBOX (INBOX DEFAULT) "
	print "\tlist_all: lists unreaded messages from all MAILBOX"
	print "\tmessages [FILTER] [MAILBOX]: list messages in [MAILBOX] with [FILTER]"
	print "\tread ID [MAILBOX]: view the ID message in the MAILBOX"
	print "\tmailbox MAILBOX: set default mailbox"
	print "\tsend TO SUBJECT: send email to TO address with SUBJECT"
#MAIN PROGRAM

signal.signal(signal.SIGINT, ctrl_c)
histfile = os.path.join(os.environ["HOME"], ".gmailhyst")
try:
	#ENABLE COMPLETION
	#readline.parse_and_bind("tab: complete")
	readline.read_history_file(histfile)
except IOError:
	pass
atexit.register(readline.write_history_file, histfile)

commands_imap = {
	'unread':'#unread',
	'list_all': 'list_all',
	'messages': '#list_messages',
	'read': '#view_message',
	'mailbox': '!1set_mailbox'
}

commands_smtp = {
	'send': '!2send_mail'
}

imap = MyImap("imap.gmail.com", 993)
smtp = MySMTP("smtp.gmail.com", 465)
#smtp.send_mail()
quit=False
print "Switching to console mode, to exit type 'quit' or press CTRL+D"
while quit==False:
	try:
		#sys.stdout.flush()
	     	#command = sys.stdin.readline().strip().split()
		#if len(command) > 0
		command = raw_input("gmail> ").split()
		if len(command) == 0:
			continue

		obj = None
		commands = None
		if command[0] in commands_imap.keys():
			obj = imap
			commands=commands_imap
		elif command[0] in commands_smtp.keys():
			obj = smtp
			commands=commands_smtp

		if obj is not None :
			try:
				if commands[command[0]].find("#")>=0 and len(command) > 1:
					getattr(obj, commands[command[0]][1:])(command[1:])
				elif commands[command[0]].find("#")>=0 and len(command) == 1:
					getattr(obj, commands[command[0]][1:])()
				elif commands[command[0]].find("!")>=0 and len(command) == int(commands[command[0]][1:2])+1:
					getattr(obj, commands[command[0]][2:])(command[1:])
				elif commands[command[0]].find("#")==-1 and commands[command[0]].find("!")==-1:
					getattr(obj, commands[command[0]])()
				else:
					help()
			except Exception as e:
				print "Error[0]: %s" % e.args
		elif command[0] == 'quit':
			quit=True
		else:
			help()
	except IOError:
		continue
	except EOFError:
		print ""
		quit = True
